module GeneralCorrelation

using Distributions,DistributionsExtended
using DataFrames
#using GLM

export initSuccessProbs
#export initDummySuccessData
#export translate_cors

function initSuccessProbs(success,rhos, Fs, pF=Normal())
    # Compute correlated probabilities of success for cmpounds in each phase.
    #
    # Note that in each state, a compound can have a different probability distribution
    # and that the probability of success for a compound can be correlated with its own 
    # success in subsequent phases and with other compounds in any phase.
    # This implies a correlation matrix of size (ncompounds*nstates)x(ncompounds*nstates)
    # and a Fs matrix of size (ncompounds*nstates), assuming a successProbabilities matric
    # of the same size.
    # Some (minor) speedup could be achieved by recording the variances once and reusing 
    # them.
    #
    # Parameter definitions:
    #  success   : (global) matrix (ncompounds x nstates) to be populated w success probabilities 
    #  ncompunds : number of compounds in portfolio
    #  nstates   : number of transitionable states (prob actual number - 1 due to residual)
    #  rhos      : (global) matrix (ncompounds*nstates)x(ncompounds*nstates) of pairwise correlations
    #  Fs        : (global) matrix (ncompounds x nstates) of parameterized probability distribuions
    #  pF        : (global) probability distribution for inducing correlation
    #
    # Structure of correlation matrix, etc. is that
    #   information for compound i, state j info is stored in matrix row i*(nstates-1)+j
    #
    nCompStates=size(rhos,1) #number of compound states, i.e., rows of cor matrix
    nstates = size(success,2)
    U = chol(rhos,:U)
    Z = rand(Normal(),(1,nCompStates))*U
    
    jj=1
    ii=1
    for i = 1:nCompStates
        if ndims(Fs)==2
            Fs[ii,jj]
            success[ii,jj]=translate_rv(Z[i],Fs[ii,jj],pF)
        else
            success[ii,jj]=translate_rv(Z[i],Fs[jj],pF)
        end
        if jj==nstates
            ii+=1
            jj=1
        else 
            jj+=1
        end
        
    end 
    return success
end

# ======================================

function translate_rv(Z,qF,pF=Normal()) 
    p=cdf(pF,Z)       
    Y=quantile(qF,p)
    return(Y::Float64)
end
#=
# ======================================

### MAY NOT WORK ON GENERAL CASES
function translate_cors(rhos,qFs,pF=Normal(0,1),echo=false)

    nrhos=7
    nrand=10000
    incr=0.05

    nvar  = size(rhos,1)
    dfCol1 = zeros(Float64,nrhos)
    dfCol2 = zeros(Float64,nrhos)

    X1    = zeros(Float64,nrand)
    X2    = zeros(Float64,nrand)

    Fs_list=Array(Any, nvar)
    nStates=size(qFs,2)
    nComp=size(qFs,1)
    rhos_s=eye(Float64,nComp*nStates)

    ii=1
    for(i = 1:nComp )
        for(j=1:nStates)
            Fs_list[ii]=qFs[i,j]
            ii+=1
        end
    end


    rho_s = .0

    for i = 1:(nvar-1) #change to varstates
        for j = (i+1):nvar                           #Calc adjusted correlations so that realized cor of transformed vars=target corr
            rho=rhos[i,j] 
            # println("rhos[",i,j,"]=",rho)            #Need to make this search much more efficient, as need to do it reasonab n*(n-1) times.        
            rho_c=max(rho-2*incr,max(rho-incr,0))    #That said, it only runs between 6 and 15 times per correlation.
            if(rho_c==0)
                rho_c=rho 
            end
            k=1
            if rho==0
                r=0
            else
                while k<=nrhos 
                    Z = rand(Normal(0,sqrt(rho_c)),nrand)
                    Y1= rand(Normal(0,sqrt(1-rho_c)),nrand)
                    Y2= rand(Normal(0,sqrt(1-rho_c)),nrand)
                    for ii = 1:nrand
                        Z1= Z[ii]+Y1[ii]
                        Z2= Z[ii]+Y2[ii]
                        X1[ii]=translate_rv(Z1,Fs_list[i],pF)
                        X2[ii]=translate_rv(Z2,Fs_list[j],pF)
                    end

                    rho_s=cor(X1,X2)
                    dfCol1[k]=rho_c
                    dfCol2[k]=rho_s
                    rho_c=rho_c+incr
                    k=k+1
                end  
                
                df = DataFrame(rhoC = dfCol1,rhoS=dfCol2)
                if(echo) 
                    println(df)
                end
                model= fit(LinearModel, rhoC~rhoS,df)
                #model=glm(rho_c ~ rho_s,df,data,Normal(),IdentityLink())
                if echo
                    print(model)
                end
                df2=DataFrame(rhoS=rho,rhoC=1)
                #r=predict(model,[1 rho_c=0 rho_s=rho])
                res=predict(model,df2)
                r=res[1]
                r=max(r,0)
            end
            rhos_s[i,j]=r
            rhos_s[j,i]=r
        end
    end
    return(rhos_s)
end

# ======================================

###USED FOR TESTING ONLY
function initDummySuccessData(ncompounds,nstates)
    nCompStates=ncompounds*nstates
    rhos=eye(Float64,nCompStates)
    for i = 1:nCompStates
        for j = i:nCompStates
            rhos[i,j]=rand()/nCompStates
            rhos[j,i]=rhos[i,j]
        end
    end
    for i=1:nCompStates
        rhos[i,i]=1
    end

    success=Array(Float64,ncompounds,nstates)
    Fs=Array(Any,ncompounds, nstates)
    for i = 1:ncompounds
        for j = 1:nstates
            if iseven(i) 
                Fs[i,j]=Beta(2*i,(j+2i))
                #Fs[i,j]=LogNormal(2*i,rand())
            else 
                Fs[i,j]=Beta(2*i,(j+2i*10))
                #Fs[i,j]=LogNormal(2*i,rand())
            end
        end
    end
    return rhos,success,Fs
end
=#
end
# ======================================
