module ParamUtilitiesOriginal

using Roots

using Distributions
using DistributionsExtended

export createProbMat,createPricingMat,createPricingMatSimple

export  calCappedLN,convertLNtoGeneralized,convertLNtoCappedLN,calBudget

export convertDistribution!

export calUpfront

export quickTimeDist


###Tri-diagonal, assumes censoring has been handled externally.
function createProbMat(steady,dur,STEPS_YR)
	n=length(steady)
	prob = zeros(n+2,n+2)
	prob[1,1]=1
	prob[n+2,n+2]=1
	for i = 1:n
		prob[i+1,i+1]=(dur[i]*STEPS_YR-1)/(dur[i]*STEPS_YR)
		prob[i+1,i+2]=(1-prob[i+1,i+1])*steady[i]
		prob[i+1,1]=1-sum(prob[i+1,:])
	end	
	prob
end

##Takes in upfront and value parameters directly - no discount factor needed
function createPricingMatSimple(value,cost,upfront,milestone,steady,target;
						vstdratio=2240.0/1870.0,
						vmx = [0, 0, 250, 500, 1000, 5000, 5000], #maximum of valuation at phase
						cstdratio = [0, 5.2/4.8, 15.2/12.8, 23.5/22.1, 86.3/60.6, 0, 0], ##sd/mean ratio of phase costs
						cmx = [0, 20, 20, 100, 500, 0, 0])

		n = length(value)
		if (length(milestone)!=n || length(cost)!=n || length(upfront)!=n || length(steady)!=n)
			error("Arguments not of equal length")
		end
		
		stdvalue = value.*vstdratio
		stdcost  =cost.*cstdratio
		
		vmu,vsigma = calCappedLN(value,stdvalue,vmx)
		cmu,csigma = calCappedLN(cost,stdcost,cmx)
		
		budget = calBudget(cost,steady,milestone,target)
		mat=hcat(vmu,vsigma,vmx,upfront,milestone,cmu,csigma,cmx,budget)
end

function calBudget(cost,steady,milestone,target)
	calBudget(Float64,cost,steady,milestone,target)
end
function calBudget(T,cost,steady,milestone,target)
	##Calculate future costs for budgeting.
	n = length(cost)
	futureCost = zeros(T,n)
	for i = n:-1:1
		if i<n
			futureCost[i]=(futureCost[i+1]+cost[i+1])*steady[i]
		end
	end
	
	##Calculate Budget
	budget = zeros(T,n)
	for i = (target-2):-1:1
		budget[i] = steady[i]*(cost[i+1]+budget[i+1]+milestone[i+1])
	end
	#budget += upfront
	return budget	
end

function calCappedLN(avg,std,max,fixedvsigma=NaN)
		n=length(avg)
		if(length(std)!=n || length(max)!=n)	
			error("Error in conversion lengths are not equal.")
		end	
		vmu=zeros(n)
		vsigma=zeros(n)
		##Convert other phases using caps.
		for i = n:-1:1
			avg[i] <= 0 && max[i] > 0 && error("Incompatible: zero average and positive cap at index $i.") 
			vsigma[i]=sqrt(log(1+std[i]^2/avg[i]^2))
			vmu[i]= log(avg[i])-0.5*vsigma[i]^2 
			if !isnan(fixedvsigma)
				vsigma[i]=fixedvsigma
			end
			##Convert using cap and assuming sigma is fixed.
			vmu[i]=convertLNtoCappedLN(vmu[i],vsigma[i],max[i],avg[i])
		end
		return vmu,vsigma
end


###Optional keyword arguments control the construction
function createPricingMat(appValue,minValue,costs,disc,dur,steady,target;
						vstdratio=2240.0/1870.0,
						vsigma_fixed=0.939, ##Fixed vsigma set to NaN always
						vmx = [100, 250, 500, 1000, 5000, 5000], #maximum of valuation at phase
						cstdratio = [5.2/4.8, 15.2/12.8, 23.5/22.1, 86.3/60.6, 0], ##sd/mean ratio of phase costs
						cmx = [20, 20, 100, 500, 0, 0],
						milestone  = [1, 3.75, 10, 37.6,0,0],
						upfront = [],
						value = [])
		n = length(disc)
		
		value = minValue
		value[n+1] = appValue		
		stdvalue=zeros(n+1)
		stdvalue[n+1] = appValue*vstdratio
		
		stdcost = zeros(n+1)
		stdcost[1:n]=costs[1:n].*cstdratio[1:n]
		
		disc_total = ones(n+1)
		futureCost = zeros(n+1)
		
		upfront_flag = !isempty(upfront)
		if !upfront_flag 
			upfront = ones(n+1)
		end
		
		
		for i = n:-1:1
			disc_total[i]=1/(1+disc[i])^dur[i]*steady[i]
			if i<n
				futureCost[i]=(futureCost[i+1]+costs[i+1])*steady[i]
			end
		
			value[i]=max(appValue*prod(disc_total[i:n])-futureCost[i],value[i])	
			stdvalue[i]=value[i]*vstdratio
		end


		if !upfront_flag 
			upfront=max(value-[costs;0],upfront)
		end
		
		budget = zeros(n+1)
		for i = (target-2):-1:1
			budget[i] = steady[i]*(costs[i+1]+budget[i+1]+milestone[i+1])
		end
		#budget += upfront

		vmu = zeros(n+1)
		vsigma = zeros(n+1)
		csigma = zeros(n+1)
		cmu = zeros(n+1)

		vmu,vsigma = calCappedLN(value,stdvalue,vmx)
		if !isnan(vsigma_fixed) 
			vsigma[:]=vsigma_fixed
		end
		cmu,csigma = calCappedLN([costs;0],stdcost,cmx) ##HACK TO CHECK/FIX

		mat=vcat([1 0 0 0 0 0 0 0 0],hcat(vmu,vsigma,vmx,upfront,milestone,cmu,csigma,cmx,budget))
end



function convertLNtoCappedLN(vmu,vsigma,vmx,value)
		if vmx == 0
			return 0
		elseif vsigma == 0
			return min(vmu,log(vmx))
		end
		vmu=fzero(x-> errorCapped(x,vsigma,vmx,value),vmu)
end

function errorCapped(vmu,vsigma,vmx,value)
	if(vsigma>0)
		P = cdf(Normal(vmu[1],vsigma),log(vmx))
		value-vmx*(1-P)-temp(vmu[1],vsigma,vmx)*P
	else
		value-min(exp(vmu),vmx)
	end
end

function temp(vmu,vsigma,vmx)
	vmx_scaled = (log(vmx)-vmu)/vsigma
	g = exp(vmu+vsigma^2/2)*cdf(Normal(),vmx_scaled-vsigma)/cdf(Normal(),vmx_scaled)
end

##Convenience function using only first moment information and semi-qualitative
## second moment assumptions
function quickTimeDist(timeDist,dur)
	if (timeDist=="Geometric")
		dist = Geometric(1/(2*dur)) ##should be used with Geometric mode
    elseif (timeDist=="LogNormal")
        v = dur.^2/4 ##somewhat arbitrary
        mu = log(dur.^2./sqrt(v+dur.^2))
        sigma = sqrt(log(1+v./dur.^2))
        dist = LogNormal(mu,sigma)
    elseif (timeDist=="Constant")
    	dist = DiscreteUniform(dur[i],dur[i])
    elseif (timeDist=="Uniform")
        width = dur[i]/2*sqrt(3)
        dist = Uniform(dur[i]-width,dur[i]+width)
    elseif (timeDist=="TruncatedNormal")
    	v = dur.^2/4 ##somewhat arbitrary
    	opt = Opt(:LN_NELDERMEAD,2)
    	lower_bounds!(opt,[0,0.01])
        xtol_rel!(opt,1E-4)
        func = (x,grad)->100*(dur-mean(Truncated(Normal(x[1],x[2]),0,Inf)))^2+(sqrt(v)-std(Truncated(Normal(x[1],x[2]),0,Inf)))^2
        min_objective!(opt,func)
        (minf,minx,ret)=optimize(opt,[dur,v])                 
        dist = Truncated(Normal(minx[1],minx[2]),0,Inf)
    end
end

##Sample to estimate 1st and 2nd moments
##FIt to two moments to the input Distribution
##Currently handles Truncated Normal, Uniform, Gamma, Pareto, Weibull and CLN (the default)
function convertLNtoGeneralized(vmu,vsigma,vmx,Distribution="CLN")
		if vmx>0
			mu=mean(min(exp(vmu+vsigma*randn(1000000)),vmx))
			sigma=sqrt(var(min(exp(vmu+vsigma*randn(1000000)),vmx)))
			if vsigma <2*eps()
				dist = DiscreteUniform(exp(vmu),exp(vmu))
			elseif Distribution=="TN"
				opt = Opt(:LN_NELDERMEAD,2)
				lower_bounds!(opt,[0,0.01])
				xtol_rel!(opt,1E-4)
				min_objective!(opt,func)
				(minf,minx,ret)=optimize(opt,[mu,sigma])
				dist = Truncated(Normal(minx[1],minx[2]),0,Inf)
			elseif Distribution == "Uniform"
				dist = fit(Uniform,min(exp(vmu+vsigma*randn(1000000))))
				w = sigma*sqrt(3)
				dist = Uniform(mu-w,mu+w)
			elseif Distribution == "Gamma"
				opt = Opt(:LN_NELDERMEAD,2)
				lower_bounds!(opt,[0.001,0.001])
				xtol_rel!(opt,1E-4)
				func = (x,grad)->100*(mu-mean(Gamma(x[1],x[2])))^2+(sigma-std(Gamma(x[1],x[2])))^2
				min_objective!(opt,func)

				(minf,minx,ret)=optimize(opt,[1,mu])
				dist = Gamma(minx[1],minx[2])
			elseif Distribution == "Pareto"
				opt = Opt(:LN_NELDERMEAD,2)
				lower_bounds!(opt,[1.001,2])
				xtol_rel!(opt,1E-4)
				func = (x,grad)->100*(mu-mean(Pareto(x[1],x[2])))^2+(sigma-std(Pareto(x[1],x[2])))^2
				min_objective!(opt,func)

				(minf,minx,ret)=optimize(opt,[5,mu])
				dist = Pareto(minx[1],minx[2])
			elseif Distribution == "Weibull"
				opt = Opt(:LN_NELDERMEAD,2)
				lower_bounds!(opt,[0.01,0.01])
				xtol_rel!(opt,1E-4)
				func = (x,grad)->100*(mu-mean(Weibull(x[1],x[2])))^2+(sigma-std(Weibull(x[1],x[2])))^2
				min_objective!(opt,func)

				(minf,minx,ret)=optimize(opt,[5,mu])
				dist = Weibull(minx[1],minx[2])
			elseif Distribution == "CLN"
				dist = Bounded(LogNormal(vmu,vsigma),0,vmx)
			else
				error("Distribution not supported.")
			end
		else
			dist = DiscreteUniform(0,0)
		end
end

function convertDistribution!{D<:UnivariateDistribution}(dist::Array{UnivariateDistribution,1},dt::Type{D})
		for i = 1:length(dist)
			if !isdefined(dist,i)
				continue
			end
			samples = rand(dist[i],1000000)
			dist[i] = fit(dt,samples)
		end
		return dist
end


end
