

immutable Bounded{D<:UnivariateDistribution} <: ContinuousUnivariateDistribution
    unBounded::D      # the original distribution (unBounded)
    l::Float64 # loocation parameter
    u::Float64
    lcdf::Float64       # cdf of lower bound
    ucdf::Float64       # cdf of upper bound
end

function Bounded(d::ContinuousUnivariateDistribution, l::Float64, u::Float64)
    l < u || error("lower bound should be less than upper bound.")
    lcdf = isinf(l) ? 0.0 : cdf(d, l)
    ucdf = isinf(u) ? 1.0 : cdf(d, u)
        
    Bounded{typeof(d)}(d, l, u, lcdf, ucdf)
end

Bounded(d::ContinuousUnivariateDistribution, l::Real, u::Real) = Bounded(d, float64(l), float64(u))
### range and support

minimum(d::Bounded) = max(minimum(d.unBounded),d.l)
maximum(d::Bounded) = min(maximum(d.unBounded),d.u)

insupport(d::Bounded, x::Real) = l<= x <=u && insupport(d.unBounded, x)

### evaluation

pdf(d::Bounded, x::Float64) = 
    d.l<= x <= d.u ?  
        x==l ? d.lcdf : 
        x==u ? 1-d.ucdf :
        pdf(d.unBounded,x) :
        0.0

logpdf(d::Bounded, x::Float64) = log(pdf(d,x))

cdf(d::Bounded, x::Float64) = x < d.l ? 0.0 : (x > d.u ? 1.0 : cdf(d.unBounded,x))

logcdf(d::Bounded, x::Float64) = log(cdf(d,x))

quantile(d::Bounded, p::Float64) = p <= d.lcdf ? d.l : p >= d.ucdf ? d.u : quantile(d.unBounded,p)

mean(d::Bounded) = d.l*d.lcdf+d.u*(1-d.ucdf)+mean(Truncated(d.unBounded,d.l,d.u))*(d.ucdf-d.lcdf)
#var(d::Bounded) = var(d.unBounded)
#mode(d::Bounded) = mode(d.unBounded)+d.l
#modes(d::Bounded) = modes(d.unBounded)+d.l
#skewness(d::Bounded) = skewness(d.unBounded)
#kurtosis(d::Bounded) = kurtosis(d.unBounded)


## random number generation

function rand(d::Bounded)
    return min(max(rand(d.unBounded),d.l),d.u)
end

## show

function show(io::IO, d::Bounded) 
    print(io, "Bounded(")
    d0 = d.unBounded
    uml, namevals = _use_multline_show(d0)
    uml ? show_multline(io, d0, namevals) : 
          show_oneline(io, d0, namevals)
    print(io, ", bounds=($(d.l), $(d.u))") 
    uml && println(io)
end

_use_multline_show(d::Bounded) = _use_multline_show(d.unBounded)
